@@ -1,15 +1,21 @@
 import math
+import os
 import unittest
 import sys
 import _ast
-from test import test_support
-from test import script_helper
-import os
 import tempfile
-import textwrap
+import types
+from test import support
+from test.support import script_helper
 
 class TestSpecifics(unittest.TestCase):
 
+    def compile_single(self, source):
+        compile(source, "<single>", "single")
+
+    def assertInvalidSingle(self, source):
+        self.assertRaises(SyntaxError, self.compile_single, source)
+
     def test_no_ending_newline(self):
         compile("hi", "<test>", "exec")
         compile("hi\r", "<test>", "exec")
@@ -26,31 +32,19 @@ class TestSpecifics(unittest.TestCase):
     def test_debug_assignment(self):
         # catch assignments to __debug__
         self.assertRaises(SyntaxError, compile, '__debug__ = 1', '?', 'single')
-        import __builtin__
-        prev = __builtin__.__debug__
-        setattr(__builtin__, '__debug__', 'sure')
-        setattr(__builtin__, '__debug__', prev)
+        import builtins
+        prev = builtins.__debug__
+        setattr(builtins, '__debug__', 'sure')
+        setattr(builtins, '__debug__', prev)
 
     def test_argument_handling(self):
         # detect duplicate positional and keyword arguments
         self.assertRaises(SyntaxError, eval, 'lambda a,a:0')
         self.assertRaises(SyntaxError, eval, 'lambda a,a=1:0')
         self.assertRaises(SyntaxError, eval, 'lambda a=1,a=1:0')
-        try:
-            exec 'def f(a, a): pass'
-            self.fail("duplicate arguments")
-        except SyntaxError:
-            pass
-        try:
-            exec 'def f(a = 0, a = 1): pass'
-            self.fail("duplicate keyword arguments")
-        except SyntaxError:
-            pass
-        try:
-            exec 'def f(a): global a; a = 1'
-            self.fail("variable is global and local")
-        except SyntaxError:
-            pass
+        self.assertRaises(SyntaxError, exec, 'def f(a, a): pass')
+        self.assertRaises(SyntaxError, exec, 'def f(a = 0, a = 1): pass')
+        self.assertRaises(SyntaxError, exec, 'def f(a): global a; a = 1')
 
     def test_syntax_error(self):
         self.assertRaises(SyntaxError, compile, "1+*3", "filename", "exec")
@@ -59,60 +53,7 @@ class TestSpecifics(unittest.TestCase):
         self.assertRaises(SyntaxError, compile, "f(None=1)", "<string>", "exec")
 
     def test_duplicate_global_local(self):
-        try:
-            exec 'def f(a): global a; a = 1'
-            self.fail("variable is global and local")
-        except SyntaxError:
-            pass
-
-    def test_exec_functional_style(self):
-        # Exec'ing a tuple of length 2 works.
-        g = {'b': 2}
-        exec("a = b + 1", g)
-        self.assertEqual(g['a'], 3)
-
-        # As does exec'ing a tuple of length 3.
-        l = {'b': 3}
-        g = {'b': 5, 'c': 7}
-        exec("a = b + c", g, l)
-        self.assertNotIn('a', g)
-        self.assertEqual(l['a'], 10)
-
-        # Tuples not of length 2 or 3 are invalid.
-        with self.assertRaises(TypeError):
-            exec("a = b + 1",)
-
-        with self.assertRaises(TypeError):
-            exec("a = b + 1", {}, {}, {})
-
-        # Can't mix and match the two calling forms.
-        g = {'a': 3, 'b': 4}
-        l = {}
-        with self.assertRaises(TypeError):
-            exec("a = b + 1", g) in g
-        with self.assertRaises(TypeError):
-            exec("a = b + 1", g, l) in g, l
-
-    def test_nested_qualified_exec(self):
-        # Can use qualified exec in nested functions.
-        code = ["""
-def g():
-    def f():
-        if True:
-            exec "" in {}, {}
-        """, """
-def g():
-    def f():
-        if True:
-            exec("", {}, {})
-        """, """
-def g():
-    def f():
-        if True:
-            exec("", {})
-        """]
-        for c in code:
-            compile(c, "<code>", "exec")
+        self.assertRaises(SyntaxError, exec, 'def f(a): global a; a = 1')
 
     def test_exec_with_general_mapping_for_locals(self):
 
@@ -129,37 +70,27 @@ def g():
 
         m = M()
         g = globals()
-        exec 'z = a' in g, m
+        exec('z = a', g, m)
         self.assertEqual(m.results, ('z', 12))
         try:
-            exec 'z = b' in g, m
+            exec('z = b', g, m)
         except NameError:
             pass
         else:
             self.fail('Did not detect a KeyError')
-        exec 'z = dir()' in g, m
+        exec('z = dir()', g, m)
         self.assertEqual(m.results, ('z', list('xyz')))
-        exec 'z = globals()' in g, m
+        exec('z = globals()', g, m)
         self.assertEqual(m.results, ('z', g))
-        exec 'z = locals()' in g, m
+        exec('z = locals()', g, m)
         self.assertEqual(m.results, ('z', m))
-        try:
-            exec 'z = b' in m
-        except TypeError:
-            pass
-        else:
-            self.fail('Did not validate globals as a real dict')
+        self.assertRaises(TypeError, exec, 'z = b', m)
 
         class A:
             "Non-mapping"
             pass
         m = A()
-        try:
-            exec 'z = a' in g, m
-        except TypeError:
-            pass
-        else:
-            self.fail('Did not validate locals as a mapping')
+        self.assertRaises(TypeError, exec, 'z = a', g, m)
 
         # Verify that dict subclasses work as well
         class D(dict):
@@ -168,11 +99,12 @@ def g():
                     return 12
                 return dict.__getitem__(self, key)
         d = D()
-        exec 'z = a' in g, d
+        exec('z = a', g, d)
         self.assertEqual(d['z'], 12)
 
     def test_extended_arg(self):
         longexpr = 'x = x or ' + '-x' * 2500
+        g = {}
         code = '''
 def f(x):
     %s
@@ -191,39 +123,11 @@ def f(x):
         # EXTENDED_ARG/JUMP_ABSOLUTE here
     return x
 ''' % ((longexpr,)*10)
-        exec code
-        self.assertEqual(f(5), 0)
-
-    def test_complex_args(self):
-
-        with test_support.check_py3k_warnings(
-                ("tuple parameter unpacking has been removed", SyntaxWarning)):
-            exec textwrap.dedent('''
-        def comp_args((a, b)):
-            return a,b
-        self.assertEqual(comp_args((1, 2)), (1, 2))
-
-        def comp_args((a, b)=(3, 4)):
-            return a, b
-        self.assertEqual(comp_args((1, 2)), (1, 2))
-        self.assertEqual(comp_args(), (3, 4))
-
-        def comp_args(a, (b, c)):
-            return a, b, c
-        self.assertEqual(comp_args(1, (2, 3)), (1, 2, 3))
-
-        def comp_args(a=2, (b, c)=(3, 4)):
-            return a, b, c
-        self.assertEqual(comp_args(1, (2, 3)), (1, 2, 3))
-        self.assertEqual(comp_args(), (2, 3, 4))
-        ''')
+        exec(code, g)
+        self.assertEqual(g['f'](5), 0)
 
     def test_argument_order(self):
-        try:
-            exec 'def f(a=1, (b, c)): pass'
-            self.fail("non-default args after default")
-        except SyntaxError:
-            pass
+        self.assertRaises(SyntaxError, exec, 'def f(a=1, b): pass')
 
     def test_float_literals(self):
         # testing bad float literals
@@ -248,29 +152,26 @@ if 1:
         s256 = "".join(["\n"] * 256 + ["spam"])
         co = compile(s256, 'fn', 'exec')
         self.assertEqual(co.co_firstlineno, 257)
-        self.assertEqual(co.co_lnotab, '')
+        self.assertEqual(co.co_lnotab, bytes())
 
     def test_literals_with_leading_zeroes(self):
         for arg in ["077787", "0xj", "0x.", "0e",  "090000000000000",
                     "080000000000000", "000000000000009", "000000000000008",
                     "0b42", "0BADCAFE", "0o123456789", "0b1.1", "0o4.2",
-                    "0b101j2", "0o153j2", "0b100e1", "0o777e1", "0o8", "0o78"]:
+                    "0b101j2", "0o153j2", "0b100e1", "0o777e1", "0777",
+                    "000777", "000000000000007"]:
             self.assertRaises(SyntaxError, eval, arg)
 
-        self.assertEqual(eval("0777"), 511)
-        self.assertEqual(eval("0777L"), 511)
-        self.assertEqual(eval("000777"), 511)
         self.assertEqual(eval("0xff"), 255)
-        self.assertEqual(eval("0xffL"), 255)
-        self.assertEqual(eval("0XfF"), 255)
         self.assertEqual(eval("0777."), 777)
         self.assertEqual(eval("0777.0"), 777)
         self.assertEqual(eval("000000000000000000000000000000000000000000000000000777e0"), 777)
         self.assertEqual(eval("0777e1"), 7770)
         self.assertEqual(eval("0e0"), 0)
-        self.assertEqual(eval("0000E-012"), 0)
+        self.assertEqual(eval("0000e-012"), 0)
         self.assertEqual(eval("09.5"), 9.5)
         self.assertEqual(eval("0777j"), 777j)
+        self.assertEqual(eval("000"), 0)
         self.assertEqual(eval("00j"), 0j)
         self.assertEqual(eval("00.0"), 0)
         self.assertEqual(eval("0e3"), 0)
@@ -279,38 +180,33 @@ if 1:
         self.assertEqual(eval("090000000000000e0"), 90000000000000.)
         self.assertEqual(eval("090000000000000e-0"), 90000000000000.)
         self.assertEqual(eval("090000000000000j"), 90000000000000j)
-        self.assertEqual(eval("000000000000007"), 7)
         self.assertEqual(eval("000000000000008."), 8.)
         self.assertEqual(eval("000000000000009."), 9.)
         self.assertEqual(eval("0b101010"), 42)
         self.assertEqual(eval("-0b000000000010"), -2)
         self.assertEqual(eval("0o777"), 511)
         self.assertEqual(eval("-0o0000010"), -8)
-        self.assertEqual(eval("020000000000.0"), 20000000000.0)
-        self.assertEqual(eval("037777777777e0"), 37777777777.0)
-        self.assertEqual(eval("01000000000000000000000.0"),
-                         1000000000000000000000.0)
 
     def test_unary_minus(self):
         # Verify treatment of unary minus on negative numbers SF bug #660455
-        if sys.maxint == 2147483647:
+        if sys.maxsize == 2147483647:
             # 32-bit machine
             all_one_bits = '0xffffffff'
-            self.assertEqual(eval(all_one_bits), 4294967295L)
-            self.assertEqual(eval("-" + all_one_bits), -4294967295L)
-        elif sys.maxint == 9223372036854775807:
+            self.assertEqual(eval(all_one_bits), 4294967295)
+            self.assertEqual(eval("-" + all_one_bits), -4294967295)
+        elif sys.maxsize == 9223372036854775807:
             # 64-bit machine
             all_one_bits = '0xffffffffffffffff'
-            self.assertEqual(eval(all_one_bits), 18446744073709551615L)
-            self.assertEqual(eval("-" + all_one_bits), -18446744073709551615L)
+            self.assertEqual(eval(all_one_bits), 18446744073709551615)
+            self.assertEqual(eval("-" + all_one_bits), -18446744073709551615)
         else:
             self.fail("How many bits *does* this machine have???")
-        # Verify treatment of constant folding on -(sys.maxint+1)
-        # i.e. -2147483648 on 32 bit platforms.  Should return int, not long.
-        self.assertIsInstance(eval("%s" % (-sys.maxint - 1)), int)
-        self.assertIsInstance(eval("%s" % (-sys.maxint - 2)), long)
+        # Verify treatment of constant folding on -(sys.maxsize+1)
+        # i.e. -2147483648 on 32 bit platforms.  Should return int.
+        self.assertIsInstance(eval("%s" % (-sys.maxsize - 1)), int)
+        self.assertIsInstance(eval("%s" % (-sys.maxsize - 2)), int)
 
-    if sys.maxint == 9223372036854775807:
+    if sys.maxsize == 9223372036854775807:
         def test_32_63_bit_values(self):
             a = +4294967296  # 1 << 32
             b = -4294967296  # 1 << 32
@@ -321,7 +217,7 @@ if 1:
             g = +9223372036854775807  # 1 << 63 - 1
             h = -9223372036854775807  # 1 << 63 - 1
 
-            for variable in self.test_32_63_bit_values.func_code.co_consts:
+            for variable in self.test_32_63_bit_values.__code__.co_consts:
                 if variable is not None:
                     self.assertIsInstance(variable, int)
 
@@ -350,10 +246,6 @@ if 1:
             stmt += "\n"
             self.assertRaises(SyntaxError, compile, stmt, 'tmp', 'single')
             self.assertRaises(SyntaxError, compile, stmt, 'tmp', 'exec')
-        # This is ok.
-        compile("from None import x", "tmp", "exec")
-        compile("from x import None as y", "tmp", "exec")
-        compile("import None as x", "tmp", "exec")
 
     def test_import(self):
         succeed = [
@@ -392,7 +284,7 @@ if 1:
             'from sys import stdin)',
             'from sys import stdin, stdout,\nstderr',
             'from sys import stdin si',
-            'from sys import stdin,',
+            'from sys import stdin,'
             'from sys import (*)',
             'from sys import (stdin,, stdout, stderr)',
             'from sys import (stdin, stdout),',
@@ -409,30 +301,32 @@ if 1:
             f2 = lambda x=2: x
             return f1, f2
         f1, f2 = f()
-        self.assertNotEqual(id(f1.func_code), id(f2.func_code))
+        self.assertNotEqual(id(f1.__code__), id(f2.__code__))
 
     def test_lambda_doc(self):
         l = lambda: "foo"
         self.assertIsNone(l.__doc__)
 
-    @test_support.requires_unicode
     def test_encoding(self):
         code = b'# -*- coding: badencoding -*-\npass\n'
         self.assertRaises(SyntaxError, compile, code, 'tmp', 'exec')
-        code = u"# -*- coding: utf-8 -*-\npass\n"
-        self.assertRaises(SyntaxError, compile, code, "tmp", "exec")
-        code = 'u"\xc2\xa4"\n'
-        self.assertEqual(eval(code), u'\xc2\xa4')
-        code = u'u"\xc2\xa4"\n'
-        self.assertEqual(eval(code), u'\xc2\xa4')
-        code = '# -*- coding: latin1 -*-\nu"\xc2\xa4"\n'
-        self.assertEqual(eval(code), u'\xc2\xa4')
-        code = '# -*- coding: utf-8 -*-\nu"\xc2\xa4"\n'
-        self.assertEqual(eval(code), u'\xa4')
-        code = '# -*- coding: iso8859-15 -*-\nu"\xc2\xa4"\n'
-        self.assertEqual(eval(code), test_support.u(r'\xc2\u20ac'))
-        code = 'u"""\\\n# -*- coding: utf-8 -*-\n\xc2\xa4"""\n'
-        self.assertEqual(eval(code), u'# -*- coding: utf-8 -*-\n\xc2\xa4')
+        code = '# -*- coding: badencoding -*-\n"\xc2\xa4"\n'
+        compile(code, 'tmp', 'exec')
+        self.assertEqual(eval(code), '\xc2\xa4')
+        code = '"\xc2\xa4"\n'
+        self.assertEqual(eval(code), '\xc2\xa4')
+        code = b'"\xc2\xa4"\n'
+        self.assertEqual(eval(code), '\xa4')
+        code = b'# -*- coding: latin1 -*-\n"\xc2\xa4"\n'
+        self.assertEqual(eval(code), '\xc2\xa4')
+        code = b'# -*- coding: utf-8 -*-\n"\xc2\xa4"\n'
+        self.assertEqual(eval(code), '\xa4')
+        code = b'# -*- coding: iso8859-15 -*-\n"\xc2\xa4"\n'
+        self.assertEqual(eval(code), '\xc2\u20ac')
+        code = '"""\\\n# -*- coding: iso8859-15 -*-\n\xc2\xa4"""\n'
+        self.assertEqual(eval(code), '# -*- coding: iso8859-15 -*-\n\xc2\xa4')
+        code = b'"""\\\n# -*- coding: iso8859-15 -*-\n\xc2\xa4"""\n'
+        self.assertEqual(eval(code), '# -*- coding: iso8859-15 -*-\n\xa4')
 
     def test_subscripts(self):
         # SF bug 1448804
@@ -506,6 +400,19 @@ if 1:
         del d[..., ...]
         self.assertNotIn((Ellipsis, Ellipsis), d)
 
+    def test_annotation_limit(self):
+        # 16 bits are available for # of annotations, but only 8 bits are
+        # available for the parameter count, hence 255
+        # is the max. Ensure the result of too many annotations is a
+        # SyntaxError.
+        s = "def f(%s): pass"
+        s %= ', '.join('a%d:%d' % (i,i) for i in range(256))
+        self.assertRaises(SyntaxError, compile, s, '?', 'exec')
+        # Test that the max # of annotations compiles.
+        s = "def f(%s): pass"
+        s %= ', '.join('a%d:%d' % (i,i) for i in range(255))
+        compile(s, '?', 'exec')
+
     def test_mangling(self):
         class A:
             def f():
@@ -514,25 +421,21 @@ if 1:
                 import __mangled_mod
                 import __package__.module
 
-        self.assertIn("_A__mangled", A.f.func_code.co_varnames)
-        self.assertIn("__not_mangled__", A.f.func_code.co_varnames)
-        self.assertIn("_A__mangled_mod", A.f.func_code.co_varnames)
-        self.assertIn("__package__", A.f.func_code.co_varnames)
+        self.assertIn("_A__mangled", A.f.__code__.co_varnames)
+        self.assertIn("__not_mangled__", A.f.__code__.co_varnames)
+        self.assertIn("_A__mangled_mod", A.f.__code__.co_varnames)
+        self.assertIn("__package__", A.f.__code__.co_varnames)
 
     def test_compile_ast(self):
         fname = __file__
-        if fname.lower().endswith(('pyc', 'pyo')):
+        if fname.lower().endswith('pyc'):
             fname = fname[:-1]
         with open(fname, 'r') as f:
             fcontents = f.read()
         sample_code = [
             ['<assign>', 'x = 5'],
-            ['<print1>', 'print 1'],
-            ['<printv>', 'print v'],
-            ['<printTrue>', 'print True'],
-            ['<printList>', 'print []'],
             ['<ifblock>', """if True:\n    pass\n"""],
-            ['<forblock>', """for n in [1, 2, 3]:\n    print n\n"""],
+            ['<forblock>', """for n in [1, 2, 3]:\n    print(n)\n"""],
             ['<deffunc>', """def foo():\n    pass\nfoo()\n"""],
             [fname, fcontents],
         ]
@@ -547,7 +450,7 @@ if 1:
             self.assertEqual(co2.co_filename, '%s3' % fname)
 
         # raise exception when node type doesn't match with compile mode
-        co1 = compile('print 1', '<string>', 'exec', _ast.PyCF_ONLY_AST)
+        co1 = compile('print(1)', '<string>', 'exec', _ast.PyCF_ONLY_AST)
         self.assertRaises(TypeError, compile, co1, '<ast>', 'eval')
 
         # raise exception when node type is no start node
@@ -558,33 +461,117 @@ if 1:
         ast.body = [_ast.BoolOp()]
         self.assertRaises(TypeError, compile, ast, '<ast>', 'exec')
 
-    def test_yet_more_evil_still_undecodable(self):
-        # Issue #25388
-        src = b"#\x00\n#\xfd\n"
-        tmpd = tempfile.mkdtemp()
-        try:
+    def test_dict_evaluation_order(self):
+        i = 0
+
+        def f():
+            nonlocal i
+            i += 1
+            return i
+
+        d = {f(): f(), f(): f()}
+        self.assertEqual(d, {1: 2, 3: 4})
+
+    @support.cpython_only
+    def test_same_filename_used(self):
+        s = """def f(): pass\ndef g(): pass"""
+        c = compile(s, "myfile", "exec")
+        for obj in c.co_consts:
+            if isinstance(obj, types.CodeType):
+                self.assertIs(obj.co_filename, c.co_filename)
+
+    def test_single_statement(self):
+        self.compile_single("1 + 2")
+        self.compile_single("\n1 + 2")
+        self.compile_single("1 + 2\n")
+        self.compile_single("1 + 2\n\n")
+        self.compile_single("1 + 2\t\t\n")
+        self.compile_single("1 + 2\t\t\n        ")
+        self.compile_single("1 + 2 # one plus two")
+        self.compile_single("1; 2")
+        self.compile_single("import sys; sys")
+        self.compile_single("def f():\n   pass")
+        self.compile_single("while False:\n   pass")
+        self.compile_single("if x:\n   f(x)")
+        self.compile_single("if x:\n   f(x)\nelse:\n   g(x)")
+        self.compile_single("class T:\n   pass")
+
+    def test_bad_single_statement(self):
+        self.assertInvalidSingle('1\n2')
+        self.assertInvalidSingle('def f(): pass')
+        self.assertInvalidSingle('a = 13\nb = 187')
+        self.assertInvalidSingle('del x\ndel y')
+        self.assertInvalidSingle('f()\ng()')
+        self.assertInvalidSingle('f()\n# blah\nblah()')
+        self.assertInvalidSingle('f()\nxy # blah\nblah()')
+        self.assertInvalidSingle('x = 5 # comment\nx = 6\n')
+
+    def test_particularly_evil_undecodable(self):
+        # Issue 24022
+        src = b'0000\x00\n00000000000\n\x00\n\x9e\n'
+        with tempfile.TemporaryDirectory() as tmpd:
             fn = os.path.join(tmpd, "bad.py")
             with open(fn, "wb") as fp:
                 fp.write(src)
-            rc, out, err = script_helper.assert_python_failure(fn)
-        finally:
-            test_support.rmtree(tmpd)
-        self.assertIn(b"Non-ASCII", err)
+            res = script_helper.run_python_until_end(fn)[0]
+        self.assertIn(b"Non-UTF-8", res.err)
+
+    def test_yet_more_evil_still_undecodable(self):
+        # Issue #25388
+        src = b"#\x00\n#\xfd\n"
+        with tempfile.TemporaryDirectory() as tmpd:
+            fn = os.path.join(tmpd, "bad.py")
+            with open(fn, "wb") as fp:
+                fp.write(src)
+            res = script_helper.run_python_until_end(fn)[0]
+        self.assertIn(b"Non-UTF-8", res.err)
+
+    @support.cpython_only
+    def test_compiler_recursion_limit(self):
+        # Expected limit is sys.getrecursionlimit() * the scaling factor
+        # in symtable.c (currently 3)
+        # We expect to fail *at* that limit, because we use up some of
+        # the stack depth limit in the test suite code
+        # So we check the expected limit and 75% of that
+        # XXX (ncoghlan): duplicating the scaling factor here is a little
+        # ugly. Perhaps it should be exposed somewhere...
+        fail_depth = sys.getrecursionlimit() * 3
+        success_depth = int(fail_depth * 0.75)
+
+        def check_limit(prefix, repeated):
+            expect_ok = prefix + repeated * success_depth
+            self.compile_single(expect_ok)
+            broken = prefix + repeated * fail_depth
+            details = "Compiling ({!r} + {!r} * {})".format(
+                         prefix, repeated, fail_depth)
+            with self.assertRaises(RecursionError, msg=details):
+                self.compile_single(broken)
+
+        check_limit("a", "()")
+        check_limit("a", ".b")
+        check_limit("a", "[0]")
+        check_limit("a", "*a")
 
     def test_null_terminated(self):
         # The source code is null-terminated internally, but bytes-like
         # objects are accepted, which could be not terminated.
-        with self.assertRaisesRegexp(TypeError, "without null bytes"):
-            compile(u"123\x00", "<dummy>", "eval")
-        with test_support.check_py3k_warnings():
-            with self.assertRaisesRegexp(TypeError, "without null bytes"):
-                compile(buffer("123\x00"), "<dummy>", "eval")
-            code = compile(buffer("123\x00", 1, 2), "<dummy>", "eval")
-            self.assertEqual(eval(code), 23)
-            code = compile(buffer("1234", 1, 2), "<dummy>", "eval")
-            self.assertEqual(eval(code), 23)
-            code = compile(buffer("$23$", 1, 2), "<dummy>", "eval")
-            self.assertEqual(eval(code), 23)
+        with self.assertRaisesRegex(ValueError, "cannot contain null"):
+            compile("123\x00", "<dummy>", "eval")
+        with self.assertRaisesRegex(ValueError, "cannot contain null"):
+            compile(memoryview(b"123\x00"), "<dummy>", "eval")
+        code = compile(memoryview(b"123\x00")[1:-1], "<dummy>", "eval")
+        self.assertEqual(eval(code), 23)
+        code = compile(memoryview(b"1234")[1:-1], "<dummy>", "eval")
+        self.assertEqual(eval(code), 23)
+        code = compile(memoryview(b"$23$")[1:-1], "<dummy>", "eval")
+        self.assertEqual(eval(code), 23)
+
+        # Also test when eval() and exec() do the compilation step
+        self.assertEqual(eval(memoryview(b"1234")[1:-1]), 23)
+        namespace = dict()
+        exec(memoryview(b"ax = 123")[1:-1], namespace)
+        self.assertEqual(namespace['x'], 12)
+
 
 class TestStackSize(unittest.TestCase):
     # These tests check that the computed stack size for a code object
@@ -623,68 +610,6 @@ class TestStackSize(unittest.TestCase):
         code += "   x and x\n" * self.N
         self.check_stack_size(code)
 
-    def check_constant(self, func, expected):
-        for const in func.__code__.co_consts:
-            if repr(const) == repr(expected):
-                break
-        else:
-            self.fail("unable to find constant %r in %r"
-                      % (expected, func.__code__.co_consts))
-
-    # Merging equal constants is not a strict requirement for the Python
-    # semantics, it's a more an implementation detail.
-    @test_support.cpython_only
-    def test_merge_constants(self):
-        # Issue #25843: compile() must merge constants which are equal
-        # and have the same type.
-
-        def check_same_constant(const):
-            ns = {}
-            code = "f1, f2 = lambda: %r, lambda: %r" % (const, const)
-            exec(code, ns)
-            f1 = ns['f1']
-            f2 = ns['f2']
-            self.assertIs(f1.__code__, f2.__code__)
-            self.check_constant(f1, const)
-            self.assertEqual(repr(f1()), repr(const))
-
-        check_same_constant(None)
-        check_same_constant(0)
-        check_same_constant(0.0)
-        check_same_constant(b'abc')
-        check_same_constant('abc')
-
-    def test_dont_merge_constants(self):
-        # Issue #25843: compile() must not merge constants which are equal
-        # but have a different type.
-
-        def check_different_constants(const1, const2):
-            ns = {}
-            exec("f1, f2 = lambda: %r, lambda: %r" % (const1, const2), ns)
-            f1 = ns['f1']
-            f2 = ns['f2']
-            self.assertIsNot(f1.__code__, f2.__code__)
-            self.check_constant(f1, const1)
-            self.check_constant(f2, const2)
-            self.assertEqual(repr(f1()), repr(const1))
-            self.assertEqual(repr(f2()), repr(const2))
-
-        check_different_constants(0, 0.0)
-        check_different_constants(+0.0, -0.0)
-        check_different_constants((0,), (0.0,))
-
-        # check_different_constants() cannot be used because repr(-0j) is
-        # '(-0-0j)', but when '(-0-0j)' is evaluated to 0j: we loose the sign.
-        f1, f2 = lambda: +0.0j, lambda: -0.0j
-        self.assertIsNot(f1.__code__, f2.__code__)
-        self.check_constant(f1, +0.0j)
-        self.check_constant(f2, -0.0j)
-        self.assertEqual(repr(f1()), repr(+0.0j))
-        self.assertEqual(repr(f2()), repr(-0.0j))
-
-
-def test_main():
-    test_support.run_unittest(__name__)
 
 if __name__ == "__main__":
     unittest.main()
